from __future__ import annotations
from pathlib import Path
import re
from datetime import date, datetime
import logging

import yaml


def get_header(fn: Path) -> tuple[dict[str, str], str]:

    if not fn.is_file():
        raise FileNotFoundError(fn)

    pat = re.compile(r"^-{3}\s*\n([\S\s]+?)\n-{3}\s*\n([\S\s]+)")

    raw = fn.read_text(errors="ignore")

    mat = pat.search(raw)
    if not mat:
        return (None, None)

    header = yaml.load(mat.group(1), Loader=yaml.BaseLoader)

    if "date" not in header:
        try:
            postdate = datetime.strptime(fn.name[:10], "%Y-%m-%d")
            header["date"] = postdate.strftime("%Y-%m-%d")
        except ValueError:
            pass

    return header, mat.group(2)


def write_header(fn: Path, meta: dict[str, str], fixchar: bool):
    """
    Parameters
    ----------
    fn: pathlib.Path
        Hugo Markdown file to write
    meta: dict of str
        Jekyll metadata to convert to Hugo metadata
    fixchar: bool
        fix bad characters
    """
    with fn.open("w") as f:
        f.write("---\n")

        for key in meta:
            if not meta[key]:  # empty key
                continue

            if isinstance(meta[key], list):
                meta[key] = " ".join(meta[key])
            elif isinstance(meta[key], bool):
                meta[key] = str(meta[key]).lower()
            elif isinstance(meta[key], (date, datetime)):
                meta[key] = meta[key].strftime("%Y-%m-%d")  # type: ignore

            if key in ("categories", "tags"):
                f.write(f"{key}:\n")
                for v in meta[key].split(" "):
                    f.write(f"- {v}\n")
                continue

            if key == "redirect_from":
                f.write("aliases:\n")
                for v in meta[key].split(" "):
                    f.write(f"- {v}\n")

            if key == "published" and meta[key] == "false":
                f.write("draft: true")
                continue

            # single element keys
            if fixchar:
                line = meta[key]
                # FIXME: use str.translate
                line = line.replace(":", " -")
                line = line.replace("(", "")
                line = line.replace(")", "")

            if key in ("summary", "excerpt"):
                f.write(f"description: {line}\n")
            else:
                f.write(f"{key}: {line}\n")

            if not isinstance(meta[key], str):
                logging.warning(f"discarded {key} for {fn}")
                continue

        f.write("---\n\n")  # ensure blank line before content



"""
convert Jekyll posts to Hugo posts

This is since `hugo import jekyll` just seems broken, even for Hugo 0.54

Usage example:

    python jekyll2hugo.py ~/myJekyllSite/_posts ~/myHugoSite/content/blog
"""
from argparse import ArgumentParser
from pathlib import Path
import hugoutils


p = ArgumentParser()
p.add_argument("jekyll_posts_dir", help="path to Jekyll _posts directory")
p.add_argument("out_dir", help="directory to write converted Hugo posts")
p.add_argument("-nofix", help="do not fix bad characters (Hugo might fail)", action="store_true")
p.add_argument("-v", "--verbose", action="store_true")
p = p.parse_args()

inpath = Path(p.jekyll_posts_dir).expanduser()
if not inpath.is_dir():
    raise NotADirectoryError(inpath)

outdir = Path(p.out_dir).expanduser()
outdir.mkdir(parents=True, exist_ok=True)

jlist = inpath.rglob("*.md")
for jfn in jlist:
    hfn = hugoutils.post2hugo(jfn, outdir, not p.nofix)
    if p.verbose and hfn:
        print(jfn, "=>", hfn)



from pathlib import Path
import logging

from .header import get_header, write_header


def post2hugo(jfn: Path, outdir: Path, fixchar: bool) -> Path:
    """
    Converts Jekyll posts to Hugo format.
    Hugo's Markdown parser is significantly stricter than Jekyll's, so you will
    likely have to do some manual fixes to the Markdown syntax here and there.

    Parameters
    ----------
    jfn : Path
        Jekyll post filename--assumes date in filename
    outdir : Path
        path to write converted post to.
    fixchar : bool
        if true, fix common metadata illegal characters such as  :=>-

    Results
    -------

    outfn : Path
        filename that was converted

    outputs converted Hugo post

    only converts front matter, does not look at content at all.
    """

    hugo = outdir / jfn.name[11:]

    if hugo.is_file():
        return None

    jekyll_header, body = get_header(jfn)
    if not jekyll_header:
        logging.error(f"not converted: {jfn}")
        return None

    write_header(hugo, jekyll_header, fixchar)

    with hugo.open("a") as f:
        f.write(body)

    return hugo
